///////////////////////////////////////////////////////////////
// Copyright (c) 2003, Karl Marchand 
// All rights reserved.
// Redistribution and use in source and binary forms, with or without modification, 
// are permitted provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. 
// Redistributions in binary form must reproduce the above copyright notice, 
// this list of conditions and the following disclaimer in the documentation and/or 
// other materials provided with the distribution. 
// Neither the name of the <ORGANIZATION> nor the names of its contributors may be used to 
// endorse or promote products derived from this software without specific prior 
// written permission. 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
// OF SUCH DAMAGE.
///////////////////////////////////////////////////////////////
// ChutDownDlg.cpp : implementation file
//

#include "stdafx.h"

#include <HtmlHelp.h>
#include <process.h>

#include "ChutDown.h"
#include "ChutDownDlg.h"
#include "RegKeyEx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define MAX_HOUR 100
#define MAX_MINUTE 59

BOOL Shutdown()
{  
	// First for 9x/ME
	if(!ExitWindowsEx(EWX_POWEROFF, NULL)){
		// for NT
		HANDLE hToken;
		TOKEN_PRIVILEGES tkp;
		BOOL fResult;     
		// Get the current process token handle so we can get shutdown privilege.
		if(OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken)){
		// Get the LUID for shutdown privilege
			LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &tkp.Privileges[0].Luid);
			tkp.PrivilegeCount = 1;
			tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
			// Get shutdown privilege for this process.
			AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, (PTOKEN_PRIVILEGES) NULL, 0);
			// Cannot test the return value of AdjustTokenPrivileges.
			if(GetLastError() == ERROR_SUCCESS){
				if(!ExitWindowsEx(EWX_POWEROFF, NULL)){
					// to shutdown NT
					fResult = InitiateSystemShutdown(NULL, _T("Shuting down"), 0, FALSE, FALSE);
					if(fResult){
						// Disable shutdown privilege.
						tkp.Privileges[0].Attributes = 0;
						AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, (PTOKEN_PRIVILEGES) NULL, 0);
						if(GetLastError() == ERROR_SUCCESS)
							return TRUE;
					}
				}
			}
		}
	}
	return FALSE;

}

void ResetRegistry()
{
	CRegKeyEx cRootKey, cFreewareKey, cProductKey;

	cRootKey.Create(HKEY_CURRENT_USER, _T("Software"));
	cFreewareKey.Create((HKEY) cRootKey, _T("Freeware"));
	cProductKey.Create((HKEY)cFreewareKey, _T("Potemkine"));
	cProductKey.RecurseDeleteKey(_T("CHUTDOWN"));
}


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	BOOL	m_bUseRegistry;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	virtual BOOL OnInitDialog();
	virtual void OnOK();
	afx_msg BOOL OnHelpInfo(HELPINFO* pHelpInfo);
	afx_msg void OnResetRegistryButton();
	afx_msg void OnSound();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	m_bUseRegistry = FALSE;
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Check(pDX, IDC_REGISTRY_CHECK, m_bUseRegistry);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	ON_WM_HELPINFO()
	ON_BN_CLICKED(IDC_RESET_REGISTRY_BUTTON, OnResetRegistryButton)
	ON_BN_CLICKED(IDC_SOUND, OnSound)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CChutDownDlg dialog

CChutDownDlg::CChutDownDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CChutDownDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CChutDownDlg)
	m_iHour = 0;
	m_iMinute = 0;
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_hSmallIcon = AfxGetApp()->LoadIcon(IDI_SMALLICON);
	m_iTimer = 0;
}

void CChutDownDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CChutDownDlg)
	DDX_Text(pDX, IDC_HOUR_EDIT, m_iHour);
	DDV_MinMaxInt(pDX, m_iHour, 0, MAX_HOUR);
	DDX_Text(pDX, IDC_MINUTE_EDIT, m_iMinute);
	DDV_MinMaxInt(pDX, m_iMinute, 0, MAX_MINUTE);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CChutDownDlg, CDialog)
	//{{AFX_MSG_MAP(CChutDownDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_DESTROY()
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_BUTTON, OnButton)
	ON_NOTIFY(UDN_DELTAPOS, IDC_HOUR_SPIN, OnDeltaposHourSpin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_MINUTE_SPIN, OnDeltaposMinuteSpin)
	ON_WM_HELPINFO()
	ON_BN_CLICKED(IDC_ABOUT_BUTTON, OnAboutButton)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CChutDownDlg message handlers

BOOL CChutDownDlg::OnInitDialog()
{

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}


	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hSmallIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here
	CChutDownApp *pMyApp = (CChutDownApp *)AfxGetApp();
	m_iHour = pMyApp->GetProfileInt(_T("Countdown"), _T("Hour"), 1);
	m_iMinute = pMyApp->GetProfileInt(_T("Countdown"), _T("Minute"), 0);

	CRect windowRect;
	GetWindowRect(&windowRect);

	{
		CClientDC dc(this);
		int iMaxX = dc.GetDeviceCaps(HORZRES);
		int iMaxY = dc.GetDeviceCaps(VERTRES);

		int iX = pMyApp->GetProfileInt(_T("Position"), _T("x"), 
			(iMaxX - windowRect.Width()) / 2);
		int iY = pMyApp->GetProfileInt(_T("Position"), _T("y"), 
			(iMaxY - windowRect.Height()) / 2);


		if((iX < iMaxX) && (iY < iMaxY))
			SetWindowPos(&wndTopMost, iX, iY, 0, 0, SWP_NOSIZE | SWP_SHOWWINDOW);
		else
			SetWindowPos(&wndTopMost, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOMOVE | SWP_SHOWWINDOW);
	}

	return UpdateData(FALSE);
}

void CChutDownDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CChutDownDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hSmallIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CChutDownDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hSmallIcon;
}

void CChutDownDlg::OnDestroy() 
{
	if(m_iTimer){
		KillTimer(m_iTimer);
		m_iTimer = 0;
	}

	CRect windowRect;
	CChutDownApp *pMyApp = (CChutDownApp *)AfxGetApp();

	GetWindowRect(&windowRect);

	BOOL bUseRegistry = pMyApp->GetProfileInt(_T("Countdown"), _T("UseRegistry"), TRUE);

	if(bUseRegistry){
		pMyApp->WriteProfileInt(_T("Position"), _T("x"), windowRect.left);
		pMyApp->WriteProfileInt(_T("Position"), _T("y"), windowRect.top);
	}

	CDialog::OnDestroy();
}

void CChutDownDlg::OnTimer(UINT nIDEvent) 
{
	if(nIDEvent == m_iTimer){
		COleDateTime newTime(COleDateTime::GetCurrentTime());
		COleDateTimeSpan timeSpan = newTime - m_Time;

		CString strText;
		int iMinute = m_iHour * 60 + m_iMinute - (int)(timeSpan.GetTotalMinutes());


		if(!iMinute){
			KillTimer(m_iTimer);
			m_iTimer = 0;
			Shutdown();
		}

		int iNewMinute = iMinute % 60;
		int iHour = (iMinute - iNewMinute) / 60;


		strText.Format(_T("%d"), iHour);
		GetDlgItem(IDC_HOUR_EDIT)->SetWindowText(strText);
		strText.Format(_T("%d"), iNewMinute);
		GetDlgItem(IDC_MINUTE_EDIT)->SetWindowText(strText);
	}

	CDialog::OnTimer(nIDEvent);
}


void CChutDownDlg::OnButton() 
{

	if(m_iTimer){
		KillTimer(m_iTimer);
		m_iTimer = 0;

		GetDlgItem(IDC_BUTTON)->SetWindowText(CString((LPCTSTR) IDS_START));
		GetDlgItem(IDC_HOUR_EDIT)->EnableWindow(TRUE);
		GetDlgItem(IDC_MINUTE_EDIT)->EnableWindow(TRUE);
	}else{
		//save data
		if(UpdateData()){
			if(m_iHour || m_iMinute){
				CChutDownApp *pMyApp = (CChutDownApp *)AfxGetApp();

				BOOL bUseRegistry = pMyApp->GetProfileInt(_T("Countdown"), _T("UseRegistry"), TRUE);

				if(bUseRegistry){
					pMyApp->WriteProfileInt(_T("Countdown"), _T("Hour"), m_iHour);
					int iReturn = pMyApp->WriteProfileInt(_T("Countdown"), _T("Minute"), m_iMinute);
				}

				// change button label
				GetDlgItem(IDC_BUTTON)->SetWindowText(CString((LPCTSTR) IDS_STOP));
				// Start timer
				m_iTimer = SetTimer(1, 800, NULL);
				m_Time = COleDateTime::GetCurrentTime();

				GetDlgItem(IDC_HOUR_EDIT)->EnableWindow(FALSE);
				GetDlgItem(IDC_MINUTE_EDIT)->EnableWindow(FALSE);
			}else{
				// Immediate shutdown
				Shutdown();
			}

		}else{
			GetDlgItem(IDC_HOUR_EDIT)->EnableWindow(TRUE);
			GetDlgItem(IDC_MINUTE_EDIT)->EnableWindow(TRUE);
		}

	}
	
}

void CChutDownDlg::OnDeltaposHourSpin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	if(!m_iTimer && UpdateData()){
		NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
		
		m_iHour -= pNMUpDown->iDelta;

		if(m_iHour < 0)
			m_iHour = MAX_HOUR;

		if(m_iHour > MAX_HOUR)
			m_iHour = 0;

		UpdateData(FALSE);
	
		*pResult = 0;
	}
}

void CChutDownDlg::OnDeltaposMinuteSpin(NMHDR* pNMHDR, LRESULT* pResult) 
{
	if(!m_iTimer && UpdateData()){
		NM_UPDOWN* pNMUpDown = (NM_UPDOWN*)pNMHDR;
		
		m_iMinute -= pNMUpDown->iDelta;

		if(m_iMinute < 0)
			m_iMinute += 60;

		if(m_iMinute >= 60)
			m_iMinute -= 60;

		UpdateData(FALSE);
		*pResult = 0;
	}
}

BOOL CChutDownDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	UINT uString = 0;

	switch(pHelpInfo->iCtrlId){

		case IDC_HOUR_EDIT:
			uString = IDS_HOUR_EDIT;
		break;
		
		case IDC_HOUR_SPIN:
			uString = IDS_HOUR_SPIN;
		break;

		case IDC_MINUTE_EDIT:
			uString = IDS_MINUTE_EDIT;
		break;

		case IDC_MINUTE_SPIN:
			uString = IDS_MINUTE_SPIN;
		break;

		case IDC_BUTTON:
			uString = IDS_BUTTON;
		break;

		case IDCANCEL:
			uString = IDS_IDCANCEL;
		break;

		case IDC_ABOUT_BUTTON:
			uString = IDS_ABOUT;
		break;

		default: //do nothing
		break;

	}

	if(uString){
		HH_POPUP popupStruct;
		CString strHelp((LPCTSTR)uString);

		popupStruct.hinst = NULL;
		popupStruct.idString = 0;
		popupStruct.pt.x = pHelpInfo->MousePos.x;
		popupStruct.pt.y = pHelpInfo->MousePos.y;
		popupStruct.clrForeground = -1;
		popupStruct.clrBackground = -1;
		popupStruct.rcMargins = CRect(-1, -1, -1, -1);
		popupStruct.pszFont  = NULL;
		popupStruct.pszText = strHelp.GetBuffer(1);
		popupStruct.cbStruct = sizeof(popupStruct);

		HWND popupHwnd = HtmlHelp(GetSafeHwnd(), NULL, 
			HH_DISPLAY_TEXT_POPUP, (DWORD) &popupStruct);
	}
	
	return TRUE;
}

BOOL CChutDownDlg::PreTranslateMessage(MSG* pMsg) 
{
	if((pMsg->message == WM_KEYDOWN) && (pMsg->wParam == VK_F6)){
		// Reset registry
		
		ResetRegistry();
		return TRUE;
	}
	
	
	return CDialog::PreTranslateMessage(pMsg);
}

void CChutDownDlg::OnAboutButton() 
{
	CAboutDlg dlgAbout;
	dlgAbout.DoModal();
}

///////////////////////////////////////////////////////////////
// About dialog box

BOOL CAboutDlg::OnInitDialog() 
{
	m_bUseRegistry = AfxGetApp()->GetProfileInt(_T("Countdown"), _T("UseRegistry"), TRUE);

	CDialog::OnInitDialog();
	
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CAboutDlg::OnOK() 
{
	if(UpdateData()){

		AfxGetApp()->WriteProfileInt(_T("Countdown"), _T("UseRegistry"), m_bUseRegistry);

		CDialog::OnOK();
	}
}

BOOL CAboutDlg::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	UINT uString = 0;

	switch(pHelpInfo->iCtrlId){
		case IDC_REGISTRY_CHECK:
			uString = IDS_REGISTRY_CHECK;
		break;

		case IDC_SOUND:
			uString = IDS_SOUND;
		break;
			
		case IDC_RESET_REGISTRY_BUTTON:
			uString = IDS_RESET_REGISTRY_BUTTON;
		break;

		case IDOK:
			uString = IDS_IDCANCEL;
		break;

		default: //do nothing
		break;

	}

	if(uString){
		HH_POPUP popupStruct;
		CString strHelp((LPCTSTR)uString);

		popupStruct.hinst = NULL;
		popupStruct.idString = 0;
		popupStruct.pt.x = pHelpInfo->MousePos.x;
		popupStruct.pt.y = pHelpInfo->MousePos.y;
		popupStruct.clrForeground = -1;
		popupStruct.clrBackground = -1;
		popupStruct.rcMargins = CRect(-1, -1, -1, -1);
		popupStruct.pszFont  = NULL;
		popupStruct.pszText = strHelp.GetBuffer(1);
		popupStruct.cbStruct = sizeof(popupStruct);

		HWND popupHwnd = HtmlHelp(GetSafeHwnd(), NULL, 
			HH_DISPLAY_TEXT_POPUP, (DWORD) &popupStruct);
	}

	return TRUE;
}

void CAboutDlg::OnResetRegistryButton() 
{
	ResetRegistry();
}


void CAboutDlg::OnSound() 
{
	// call sound control panel	
	WinExec("rundll32.exe shell32.dll,Control_RunDLL mmsys.cpl @1", SW_NORMAL);
}
