///////////////////////////////////////////////////////////////
// Copyright (c) 2003, Karl Marchand 
// All rights reserved.
// Redistribution and use in source and binary forms, with or without modification, 
// are permitted provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. 
// Redistributions in binary form must reproduce the above copyright notice, 
// this list of conditions and the following disclaimer in the documentation and/or 
// other materials provided with the distribution. 
// Neither the name of the <ORGANIZATION> nor the names of its contributors may be used to 
// endorse or promote products derived from this software without specific prior 
// written permission. 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
// OF SUCH DAMAGE.
///////////////////////////////////////////////////////////////
// SplashDialog.cpp : implementation file
//

#include "stdafx.h"
#include <Vfw.h>

#include "resource.h"
#include "SplashDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define SPLASH_DELAY_MAX 4000
 
/////////////////////////////////////////////////////////////////////////////
// CDib implementation

 const  int     MAXPALCOLORS = 256;
 
 IMPLEMENT_DYNAMIC(CDib, CObject)
 
 CDib::CDib()
 {
    memset(&m_bm, 0, sizeof(m_bm));
    m_hdd = NULL;
 }
 
 CDib::~CDib()
 {
    DeleteObject();
 }
 
 //////////////////
 // Delete Object. Delete DIB and palette.
 //
 BOOL CDib::DeleteObject()
 {
    m_pal.DeleteObject();
    if (m_hdd) {
       DrawDibClose(m_hdd);
       m_hdd = NULL;
    }
    memset(&m_bm, 0, sizeof(m_bm));
    return CBitmap::DeleteObject();
 }
 
 
 //////////////////
 // Load bitmap resource. Never tested.
 //
 BOOL CDib::Load(HINSTANCE hInst, LPCTSTR lpResourceName)
 {
    return Attach(::LoadImage(hInst, lpResourceName, IMAGE_BITMAP, 0, 0,
       LR_CREATEDIBSECTION | LR_DEFAULTSIZE));
 }
 
 //////////////////
 // Attach is just like the CGdiObject version,
 // except it also creates the palette
 //
 BOOL CDib::Attach(HGDIOBJ hbm)
 {
    if (CBitmap::Attach(hbm)) {
       if (!GetBitmap(&m_bm))        // load BITMAP for speed
          return FALSE;
       m_pal.DeleteObject();         // in case one is already there
       return CreatePalette(m_pal);  // create palette
    }
    return FALSE;  
 }
 
 //////////////////
 // Get size (width, height) of bitmap.
 // extern fn works for ordinary CBitmap objects.
 //
 CSize GetBitmapSize(CBitmap* pBitmap)
 {
    BITMAP bm;

    return pBitmap->GetBitmap(&bm) ? 
		CSize(bm.bmWidth, bm.bmHeight) : 
		CSize(0,0);
 }
 
 //////////////////
 // You can use this static function to draw ordinary
 // CBitmaps as well as CDibs
 //
 BOOL DrawBitmap(CDC& dc, CBitmap* pBitmap,
    const CRect* rcDst, const CRect* rcSrc)
 {
    // Compute rectangles where NULL specified
    CRect rc;
    if (!rcSrc) {
       // if no source rect, use whole bitmap
       rc = CRect(CPoint(0,0), GetBitmapSize(pBitmap));
       rcSrc = &rc;
    }
    if (!rcDst) {
       // if no destination rect, use source
       rcDst=rcSrc;
    }
 
    // Create memory DC
    CDC memdc;
    memdc.CreateCompatibleDC(&dc);
    CBitmap* pOldBm = memdc.SelectObject(pBitmap);
 
    // Blast bits from memory DC to target DC.
    // Use StretchBlt if size is different.
    //
    BOOL bRet = FALSE;
    if (rcDst->Size()==rcSrc->Size()) {
       bRet = dc.BitBlt(rcDst->left, rcDst->top, 
          rcDst->Width(), rcDst->Height(),
          &memdc, rcSrc->left, rcSrc->top, SRCCOPY);
    } else {
       dc.SetStretchBltMode(COLORONCOLOR);
       bRet = dc.StretchBlt(rcDst->left, rcDst->top, rcDst->Width(),
          rcDst->Height(), &memdc, rcSrc->left, rcSrc->top, rcSrc->Width(),
          rcSrc->Height(), SRCCOPY);
    }
    memdc.SelectObject(pOldBm);
 
    return bRet;
 }
 
 ////////////////////////////////////////////////////////////////
 // Draw DIB on caller's DC. Does stretching from source to destination
 // rectangles. Generally, you can let the following default to zero/NULL:
 //
 //    bUseDrawDib = whether to use use DrawDib, default TRUE
 //    pPal        = palette, default=NULL, (use DIB's palette)
 //    bForeground = realize in foreground (default FALSE)
 //
 // If you are handling palette messages, you should use bForeground=FALSE,
 // since you will realize the foreground palette in WM_QUERYNEWPALETTE.
 //
 BOOL CDib::Draw(CDC& dc, const CRect* rcDst, const CRect* rcSrc,
    BOOL bUseDrawDib, CPalette* pPal, BOOL bForeground)
 {
    if (!m_hObject)
       return FALSE;
 
    // Select, realize palette
    if (pPal==NULL)            // no palette specified:
       pPal = GetPalette();    // use default
    CPalette* pOldPal = dc.SelectPalette(pPal, !bForeground);
    dc.RealizePalette();
 
    BOOL bRet = FALSE;
    if (bUseDrawDib) {
       // Compute rectangles where NULL specified
       //
       CRect rc(0,0,-1,-1); // default for DrawDibDraw
       if (!rcSrc)
          rcSrc = &rc;
       if (!rcDst)
          rcDst=rcSrc;
       if (!m_hdd)
          VERIFY(m_hdd = DrawDibOpen());
 
       // Get BITMAPINFOHEADER/color table. I copy into stack object each time.
       // This doesn't seem to slow things down visibly.
       //
       DIBSECTION ds;
       VERIFY(GetObject(sizeof(ds), &ds)==sizeof(ds));
       char buf[sizeof(BITMAPINFOHEADER) + MAXPALCOLORS*sizeof(RGBQUAD)];
       BITMAPINFOHEADER& bmih = *(BITMAPINFOHEADER*)buf;
       RGBQUAD* colors = (RGBQUAD*)(&bmih+1);
       memcpy(&bmih, &ds.dsBmih, sizeof(bmih));
       GetColorTable(colors, MAXPALCOLORS);
 
       // Let DrawDib do the work!
       bRet = DrawDibDraw(m_hdd, dc,
          rcDst->left, rcDst->top, rcDst->Width(), rcDst->Height(),
          &bmih,         // ptr to BITMAPINFOHEADER + colors
          m_bm.bmBits,   // bits in memory
          rcSrc->left, rcSrc->top, rcSrc->Width(), rcSrc->Height(),
          bForeground ? 0 : DDF_BACKGROUNDPAL);
 
    } else {
       // use normal draw function
       bRet = DrawBitmap(dc, this, rcDst, rcSrc);
    }
    if (pOldPal)
       dc.SelectPalette(pOldPal, TRUE);
    return bRet;
 }
 
 #define PALVERSION 0x300   // magic number for LOGPALETTE
 
 //////////////////
 // Create the palette. Use halftone palette for hi-color bitmaps.
 //
 BOOL CDib::CreatePalette(CPalette& pal)
 { 
    // should not already have palette
    ASSERT(pal.m_hObject==NULL);
 
    BOOL bRet = FALSE;
    RGBQUAD* colors = new RGBQUAD[MAXPALCOLORS];
    UINT nColors = GetColorTable(colors, MAXPALCOLORS);
    if (nColors > 0) {
       // Allocate memory for logical palette 
       int len = sizeof(LOGPALETTE) + sizeof(PALETTEENTRY) * nColors;
       LOGPALETTE* pLogPal = (LOGPALETTE*)new char[len];
       if (!pLogPal)
          return NULL;
 
       // set version and number of palette entries
       pLogPal->palVersion = PALVERSION;
       pLogPal->palNumEntries = nColors;
 
       // copy color entries 
       for (UINT i = 0; i < nColors; i++) {
          pLogPal->palPalEntry[i].peRed   = colors[i].rgbRed;
          pLogPal->palPalEntry[i].peGreen = colors[i].rgbGreen;
          pLogPal->palPalEntry[i].peBlue  = colors[i].rgbBlue;
          pLogPal->palPalEntry[i].peFlags = 0;
       }
 
       // create the palette and destroy LOGPAL
       bRet = pal.CreatePalette(pLogPal);
       delete [] (char*)pLogPal;
    } else {
       CWindowDC dcScreen(NULL);
       bRet = pal.CreateHalftonePalette(&dcScreen);
    }
    delete colors;
    return bRet;
 }
 
 //////////////////
 // Helper to get color table. Does all the mem DC voodoo.
 //
 UINT CDib::GetColorTable(RGBQUAD* colorTab, UINT nColors)
 {
    CWindowDC dcScreen(NULL);
    CDC memdc;
    memdc.CreateCompatibleDC(&dcScreen);
    CBitmap* pOldBm = memdc.SelectObject(this);
    nColors = GetDIBColorTable(memdc, 0, nColors, colorTab);
    memdc.SelectObject(pOldBm);
    return nColors;
 }


/////////////////////////////////////////////////////////////////////////////
// CSplashThread implementation

BEGIN_MESSAGE_MAP(CSplashThread, CWinThread)
	//{{AFX_MSG_MAP(CSplashThread)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_CLOSE_SPLASH, OnCloseSplash)
END_MESSAGE_MAP()

IMPLEMENT_DYNCREATE(CSplashThread, CWinThread)


CSplashThread::CSplashThread()
{
}

CSplashThread::~CSplashThread()
{
}

BOOL CSplashThread::InitInstance()
{
	//Attach this threads UI state to the main one, This will ensure that 
	//the activation state is managed consistenly across the two threads
	ASSERT(AfxGetApp());
	BOOL bSuccess = AttachThreadInput(m_nThreadID, AfxGetApp()->m_nThreadID, TRUE);
	if (!bSuccess)
		TRACE(_T("Failed in call to AttachThreadInput, GetLastError:%d\n"), ::GetLastError());
	
	//Create the Splash Screen UI
	BOOL bCreated = m_SplashScreen.CreateWnd();
	if(bCreated){
		m_pMainWnd = &m_SplashScreen;
		HMODULE hResourceInst = GetModuleHandle("ChutDown.exe");
		HICON hIcon = (HICON) LoadImage(hResourceInst, 
			MAKEINTRESOURCE(IDR_MAINFRAME), IMAGE_ICON, 32, 32, 0);

		m_SplashScreen.SetIcon(hIcon, FALSE);
		m_SplashScreen.SetIcon(hIcon, TRUE);
	}

	return TRUE;
}

int CSplashThread::ExitInstance() 
{
	return 0;
}

BOOL CSplashThread::PreTranslateMessage(MSG* pMsg) 
{
	if((pMsg->message == WM_KEYDOWN) && (pMsg->wParam == VK_ESCAPE)){
		HideSplash();
		return TRUE;
	}

	
	return CWinThread::PreTranslateMessage(pMsg);
}

LRESULT CSplashThread::OnCloseSplash(WPARAM, LPARAM)
{
	HideSplash();

	return 0L;
}

void CSplashThread::HideSplash()
{
	if (m_SplashScreen.GetSafeHwnd()){
		if(m_SplashScreen.m_bInitialized){
			m_SplashScreen.SetOKToClose();
			m_SplashScreen.SendMessage(WM_CLOSE);
		}else
			PostThreadMessage(WM_CLOSE_SPLASH, 0, 0);
	}
}


void CSplashThread::SetBitmapToUse(UINT nID, CWnd *pParent)
{
	m_SplashScreen.SetBitmapToUse(nID, pParent);
}


BEGIN_MESSAGE_MAP(CSplashWnd, CWnd)
	//{{AFX_MSG_MAP(CSplashWnd)
	ON_WM_PAINT()
	ON_WM_PALETTECHANGED()
	ON_WM_QUERYNEWPALETTE()
	ON_WM_CLOSE()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CSplashWnd::CSplashWnd()
{
	m_bOKToClose = FALSE;
	m_nHeight = 0;
	m_nWidth = 0;
	m_pParent = NULL;
	m_nTimerID = 0;
	m_nBitmapID = 0;
	m_bInitialized = FALSE;
}

CSplashWnd::~CSplashWnd()
{
	//destroy our invisible owner when we're done
	if (m_wndOwner.m_hWnd != NULL)
		m_wndOwner.DestroyWindow();

/*	CWnd *pMainWnd = AfxGetApp()->m_pMainWnd;
	if(pMainWnd){
		pMainWnd->SetWindowPos(&(pMainWnd->wndTopMost), 
		0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
		pMainWnd->SetWindowPos(&(pMainWnd->wndNoTopMost), 
		0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
	}*/
/*	if(m_nTimerID)
		KillTimer(m_nTimerID);*/
}

BOOL CSplashWnd::LoadBitmap()
{
	ASSERT(m_nBitmapID);
	CDib cDib;

	BOOL bLoad = cDib.Load(AfxGetResourceHandle(), m_nBitmapID);
	ASSERT(bLoad);

	// should not already have palette
	ASSERT(m_Palette.m_hObject==NULL);
	cDib.CreatePalette(m_Palette);
	CDC MemDC;
	MemDC.CreateCompatibleDC(NULL);
	CSize imageSize = GetBitmapSize(&cDib);

	CClientDC dc(this);
	m_Bitmap.CreateCompatibleBitmap(&dc, imageSize.cx, imageSize.cy);
	CBitmap *pOldBmp = MemDC.SelectObject(&m_Bitmap);
	MemDC.FillSolidRect(0, 0, imageSize.cx, imageSize.cy, RGB(0, 0, 0));
	cDib.Draw(MemDC, NULL, NULL, FALSE, &m_Palette);
	MemDC.SelectObject(pOldBmp);
	MemDC.DeleteDC();

	//Get the width and height of the DIBSection
	BITMAP bm;
	GetObject(m_Bitmap.GetSafeHandle(), sizeof(BITMAP), &bm);
	m_nHeight = bm.bmHeight;
	m_nWidth = bm.bmWidth;

	return TRUE;   
}

BOOL CSplashWnd::CreateWnd()
{                   
	//Modify the owner window of the splash screen to be an invisible WS_POPUP 
	//window so that the splash screen does not appear in the task bar
	LPCTSTR pszWndClass = AfxRegisterWndClass(0);
	VERIFY(m_wndOwner.CreateEx(0, pszWndClass, _T(""), WS_POPUP, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, 
		CW_USEDEFAULT, NULL, 0));
	
	//Create this window
	pszWndClass = AfxRegisterWndClass(0, AfxGetApp()->LoadStandardCursor(IDC_ARROW));
	VERIFY(CreateEx(0, pszWndClass, _T(""), WS_POPUP | WS_VISIBLE, CRect(0, 0, 1, 1), &m_wndOwner, NULL));

	//Load up the bitmap from file or from resource
	if(!LoadBitmap())
		return FALSE;

	SetWindowPos(&wndTopMost, 0, 0, m_nWidth, m_nHeight, SWP_NOMOVE | SWP_SHOWWINDOW);
	SetWindowPos(&wndNoTopMost, 0, 0, m_nWidth, m_nHeight, SWP_NOMOVE | SWP_SHOWWINDOW);
	CenterWindow();
	//realize the bitmap's palette into the DC
	OnQueryNewPalette();
	InvalidateRect(NULL);
	SendMessage(WM_PAINT);
	m_bInitialized = TRUE;

	HMODULE hResourceInst = GetModuleHandle("ChutDown.exe");
	HICON hIcon = (HICON) ::LoadImage(hResourceInst, 
		MAKEINTRESOURCE(IDR_MAINFRAME), IMAGE_ICON, 32, 32, 0);

	SetIcon(hIcon, FALSE);
	SetIcon(hIcon, TRUE);

	return TRUE;
}

void CSplashWnd::OnPaint()
{
	CPaintDC dc(this);
	//select the palette and bitmap to the DC
	CDC memDC;
	memDC.CreateCompatibleDC(&dc);
	CBitmap* pOldBitmap = memDC.SelectObject(&m_Bitmap);
	CPalette* pOldPalette = dc.SelectPalette(&m_Palette, FALSE);
	dc.RealizePalette();
	dc.BitBlt(0, 0, m_nWidth, m_nHeight, &memDC, 0, 0, SRCCOPY);
	memDC.SelectObject(pOldBitmap);         
	dc.SelectPalette(pOldPalette, FALSE);
}

void CSplashWnd::OnClose() 
{
	if (m_bOKToClose) 
		CWnd::OnClose();
}

BOOL CSplashWnd::SelRelPal(BOOL bForceBkgnd)
{
	if(!m_Palette.m_hObject)
		return TRUE;

	// We are going active, so realize our palette.
	CDC* pDC = GetDC();
	
	CPalette* pOldPal = pDC->SelectPalette(&m_Palette, bForceBkgnd);
	UINT nResult = pDC->RealizePalette();
	pDC->SelectPalette(pOldPal, TRUE);
	pDC->RealizePalette();
	ReleaseDC(pDC);
	// If any colors have changed or we are in the
	// background, repaint the lot.
	if (nResult || bForceBkgnd) 
		InvalidateRect(NULL, TRUE); // Repaint.

	return (BOOL) nResult; // TRUE if some colors changed.
}

void CSplashWnd::OnPaletteChanged(CWnd* pFocusWnd)
{
	// See if the change was caused by us and ignore it if not.
	if (pFocusWnd != this) 
		SelRelPal(TRUE); // Realize in the background. 
}

BOOL CSplashWnd::OnQueryNewPalette()
{
	return SelRelPal(FALSE); // Realize in the foreground.
}

void CSplashWnd::SetBitmapToUse(UINT nID, CWnd *pParent)
{
	m_sFilename.Empty();
	m_pParent = pParent;
	m_nBitmapID = nID;
}






