///////////////////////////////////////////////////////////////
// Copyright (c) 2003, Karl Marchand 
// All rights reserved.
// Redistribution and use in source and binary forms, with or without modification, 
// are permitted provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. 
// Redistributions in binary form must reproduce the above copyright notice, 
// this list of conditions and the following disclaimer in the documentation and/or 
// other materials provided with the distribution. 
// Neither the name of the <ORGANIZATION> nor the names of its contributors may be used to 
// endorse or promote products derived from this software without specific prior 
// written permission. 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
// OF SUCH DAMAGE.
///////////////////////////////////////////////////////////////
// SplashDialog.h : header file
//

#if !defined(AFX_SPLASHDIALOG_K__AC270155_92B9_11D4_9905_00104B458137__INCLUDED_)
#define AFX_SPLASHDIALOG_K__AC270155_92B9_11D4_9905_00104B458137__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define WM_CLOSE_SPLASH WM_APP + 1

///////////////// Classes //////////////////////////

// global functions for ordinary CBitmap too
//
extern CSize GetBitmapSize(CBitmap* pBitmap);
extern BOOL  DrawBitmap(CDC& dc, CBitmap* pBitmap, const CRect* rcDst=NULL, const CRect* rcSrc=NULL);
 
/////////////////////////////////////////////////////////////////////////////
// CDib
/////////////////////////////////////////////////////////////////////////////

 ////////////////
 // CDib implements Device Independent Bitmaps as a form of CBitmap. 
 //
 class CDib : public CBitmap {
 protected:
    DECLARE_DYNAMIC(CDib)
    BITMAP   m_bm;    // stored for speed
    CPalette m_pal;   // palette
    HDRAWDIB m_hdd;   // for DrawDib
 
 public:
    CDib();
    ~CDib();
 
    CSize GetSize() { return CSize(m_bm.bmWidth, m_bm.bmHeight); }
    BOOL Attach(HGDIOBJ hbm);
    BOOL Load(HINSTANCE hInst, LPCTSTR lpResourceName);
    BOOL Load(HINSTANCE hInst, UINT uID)
       { return Load(hInst, MAKEINTRESOURCE(uID)); }
 
    // Universal Draw function can use DrawDib or not.
    BOOL Draw(CDC& dc, const CRect* rcDst=NULL, const CRect* rcSrc=NULL,
       BOOL bUseDrawDib=TRUE, CPalette* pPal=NULL, BOOL bForeground=FALSE);
 
    BOOL DeleteObject();
    BOOL CreatePalette(CPalette& pal);
    CPalette* GetPalette()  { return &m_pal; }
 
    UINT GetColorTable(RGBQUAD* colorTab, UINT nColors);
 };

/////////////////////////////////////////////////////////////////////////////
// CSplashWnd
/////////////////////////////////////////////////////////////////////////////

class CSplashWnd : public CWnd
{
// Attributes
public:
	BOOL m_bOKToClose;
	BOOL m_bInitialized;
	CBitmap m_Bitmap;
	CPalette m_Palette;
	int m_nHeight;
	int m_nWidth;
	CWnd m_wndOwner;
	LPCTSTR m_pszResourceName;
	CString m_sFilename;
	CWnd *m_pParent;

protected:
	UINT m_nTimerID;
	UINT m_nBitmapID;

public:
	CSplashWnd();
	~CSplashWnd();

// Operations
	inline void SetOKToClose() { 
		m_bOKToClose = TRUE; 
	};  

	void SetBitmapToUse(UINT nID, CWnd *pParent);
	BOOL CreateWnd();
  
protected:
	//{{AFX_VIRTUAL(CSplashWnd)
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CSplashWnd)
	afx_msg void OnPaint();
	afx_msg void OnPaletteChanged(CWnd* pFocusWnd);
	afx_msg BOOL OnQueryNewPalette();
	afx_msg void OnClose();
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()

	BOOL SelRelPal(BOOL bForceBkgnd);
	BOOL LoadBitmap();
};

/////////////////////////////////////////////////////////////////////////////
// CSplashThread
/////////////////////////////////////////////////////////////////////////////

class CSplashThread : public CWinThread
{
public:
	void HideSplash();
	void SetBitmapToUse(UINT nID, CWnd *pParent);

protected:
	CSplashThread();
  virtual ~CSplashThread();

	DECLARE_DYNCREATE(CSplashThread)

	//{{AFX_VIRTUAL(CSplashThread)
	public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CSplashThread)
	//}}AFX_MSG
	afx_msg LRESULT OnCloseSplash(WPARAM, LPARAM);
	DECLARE_MESSAGE_MAP()

	CSplashWnd m_SplashScreen;
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.


#endif // !defined(AFX_SPLASHDIALOG_K__AC270155_92B9_11D4_9905_00104B458137__INCLUDED_)

