///////////////////////////////////////////////////////////////
// Copyright (c) 2003, Karl Marchand 
// All rights reserved.
// Redistribution and use in source and binary forms, with or without modification, 
// are permitted provided that the following conditions are met:
//
// Redistributions of source code must retain the above copyright notice, 
// this list of conditions and the following disclaimer. 
// Redistributions in binary form must reproduce the above copyright notice, 
// this list of conditions and the following disclaimer in the documentation and/or 
// other materials provided with the distribution. 
// Neither the name of the <ORGANIZATION> nor the names of its contributors may be used to 
// endorse or promote products derived from this software without specific prior 
// written permission. 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE 
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
// OF SUCH DAMAGE.
//

#include "stdafx.h"

#include <Mmsystem.h>
#include <Vfw.h>
#include <io.h>

#include "SplashScreen.h"
#include "RegKeyEx.h"
#include "resource.h"
#include "SplashDialog.h"


// Display a bitmap and play the specific start sound
CWinThread *RunSplashThread(CString &strTitle, CString &strAppName, UINT nID)
{
	// Register sound events - can be parameterized with control panel
	CRegKeyEx cRootKey, cEventLabelsKey, cEventsNameKey, cSchemeKey, cAppsKey, cMainAppKey, cCurrentKey;

	BOOL bDefault = (cRootKey.Create(HKEY_CURRENT_USER, _T("AppEvents")) != ERROR_SUCCESS);
	bDefault = bDefault || (cEventLabelsKey.Create((HKEY) cRootKey, _T("EventLabels")) != ERROR_SUCCESS);
	bDefault = bDefault || (cEventsNameKey.Create((HKEY)cEventLabelsKey, _T("StartSound")) != ERROR_SUCCESS);
	if(!bDefault){

		HMODULE hResourceInst = GetModuleHandle("ChutDown.exe");

		VERIFY(hResourceInst);
		TCHAR buffer[2048];
		::LoadString(hResourceInst, IDS_STARTSCREEN_EVENT, buffer, 2048);
		cEventsNameKey.SetValue(buffer);
		cEventsNameKey.Detach();
	}

	bDefault = bDefault || (cSchemeKey.Create((HKEY) cRootKey, _T("Schemes")) != ERROR_SUCCESS);
	bDefault = bDefault || (cAppsKey.Create((HKEY) cSchemeKey, _T("Apps")) != ERROR_SUCCESS);
	bDefault = bDefault || (cMainAppKey.Create((HKEY)cAppsKey, strAppName) != ERROR_SUCCESS);
	if(!bDefault)
		cMainAppKey.SetValue(strTitle);

	bDefault = bDefault || (cEventsNameKey.Create((HKEY)cMainAppKey, _T("StartSound")) != ERROR_SUCCESS);
	bDefault = bDefault || (cCurrentKey.Create((HKEY)cEventsNameKey, _T(".Current")) != ERROR_SUCCESS);
	if(!bDefault){
		cEventsNameKey.Detach();
		cCurrentKey.Detach();
		cRootKey.Detach();
	}

	// Get application standard registry values
	CRegKeyEx cCompanyKey, cSoftwareKey, cApplicationKey, cSectionKey;
	bDefault = (cRootKey.Create(HKEY_CURRENT_USER, _T("Software")) != ERROR_SUCCESS);
	bDefault = bDefault || (cCompanyKey.Create((HKEY) cRootKey, _T("Freeware")) != ERROR_SUCCESS);
	bDefault = bDefault || (cSoftwareKey.Create((HKEY)cCompanyKey, _T("Potemkine")) != ERROR_SUCCESS);
	bDefault = bDefault || (cApplicationKey.Create((HKEY) cSoftwareKey, strTitle) != ERROR_SUCCESS);
	bDefault = bDefault || (cSectionKey.Create((HKEY) cApplicationKey, _T("StartScreen")) != ERROR_SUCCESS);

	CString strBitmapName;
	BOOL bActivate = TRUE;

	strBitmapName.Empty();

	CSplashThread* pSplashThread = (CSplashThread*) AfxBeginThread(RUNTIME_CLASS(CSplashThread), 
		THREAD_PRIORITY_HIGHEST, 0, CREATE_SUSPENDED);
	if (!pSplashThread){
		TRACE0("Failed to create splash screen\n");
	}else{
		ASSERT(pSplashThread->IsKindOf(RUNTIME_CLASS(CSplashThread)));

		pSplashThread->SetBitmapToUse(nID, NULL);  
		//Resume the thread now that we have set it up
		pSplashThread->ResumeThread();  
	}

	PlaySound(NULL, 0, SND_PURGE);
	sndPlaySound(_T("StartSound"), SND_APPLICATION | SND_ALIAS | SND_ASYNC | SND_NODEFAULT);
	return pSplashThread;
}

void EndSplashThread(CWinThread *pThread)
{
	if(!pThread)
		return;

	CSplashThread *pSplashThread = (CSplashThread *)pThread;
	ASSERT(pSplashThread->IsKindOf(RUNTIME_CLASS(CSplashThread)));

	pSplashThread->HideSplash();
}
